//+-------------------------------------------------------------------+
//|                                              ExpertBuilder v1.mq4 |
//|                                     Copyright  2007, GridTraders |
//|                                        http://www.gridtraders.com |
//+-------------------------------------------------------------------+

#property copyright "Copyright  2007, GridTraders"
#property link      "http://www.gridtraders.com"


//########## EXTERNAL SETTINGS ##########

/*--- If you want to access settings from your chart through Expert>Properties>Common,
      add extern in front of the data type                                          */ 

extern double LotSize = 1.00;         // Lot size for each order 
extern double TakeProfit = 710;       // Take profit level
extern double StopLoss = 65;          // Stop loss level

extern double ExpertMagic = 1000;     // Magic number used so the expert can identify it's trades
extern string ExpertName = "Expert";  // Optional, but works similar to above and for you visually when looking at your orders   
extern double timeFrame1 = 1440;      // What time frame is this expert for? 0 for any time frame, 1440 daily, 240 four hour, 60 one hour, 30 thirty minutes, and so on 
extern bool   StackOrders = true;     // This setting allows you to place more than one long or short like order when the last one placed is in profit


extern double ProfitValueClose = 40;  // When candle turns bullish, all sells are closed EXCEPT for orders with a profit value larger than this number, vice versa for longs
extern bool   CloseOnSwitch = false;  // When candle turns bullish, all sells are closed, vice versa for longs 


extern bool    UseTimeTrader  = true; //Will trade only between the hours below
extern int     Start          = 0;    // Trading  Start Time Hour (minutes are auto set to :00 

extern bool    UseTimeExpire  = true; // Expire Pending Trades
extern int     ExpireHour     = 23;   // Hour to Expire Pending Trades
extern int     ExpireMinute   = 59;   // Minute Expire Pending Trades

bool golong = false, goshort = false;  // Long and short orders are not traded until rules firther down the expert are true


///--- Here we activate the expert advisor telling it to begin working
int init()
  {
  #property show_inputs


///--- Here we are adding copyright text to the upper portion of the active chart 
ObjectCreate("text_object",OBJ_LABEL,0,0,0);
ObjectSet("text_object",OBJPROP_XDISTANCE,220);
ObjectSet("text_object",OBJPROP_YDISTANCE,0);
ObjectSetText("text_object","ExpertBuilder  www.gridtraders.com",10,"Verdana",DodgerBlue);}


///--- The function below "when called upon" closes long orders only that are either open buys or open buy stops
void CloseLongOrders()
{
  int totalorders = OrdersTotal();
  for(int i=totalorders-1;i>=0;i--)
 {
    OrderSelect(i, SELECT_BY_POS);
    bool result = false;
    if ( OrderSymbol()==Symbol() && ( (OrderMagicNumber() == ExpertMagic )) )  
     {


//--- Here we are closing pending orders only
if ( OrderType() == OP_BUYSTOP ) result = OrderDelete( OrderTicket());

//--- Now we are closing only orders that are have a profit less than are "extern double ProfitValueClose" setting above
if ( OrderType() == OP_BUY && OrderProfit() < ProfitValueClose)  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(),MODE_BID),5,Red );

//--- If are "extern bool CloseOnSwitch" setting is true, close all open buy orders regardless of profit
if ( OrderType() == OP_BUY && CloseOnSwitch == true)  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(),MODE_BID),5,Red );
      }
  }
  return;
}


///--- The function below "when called upon" closes short orders only that are either open sells or open sell stops
void CloseShortOrders()
{
  int totalorders = OrdersTotal();
  for(int i=totalorders-1;i>=0;i--)
 {
    OrderSelect(i, SELECT_BY_POS);
    bool result = false;
    if ( OrderSymbol()==Symbol() && ( (OrderMagicNumber() == ExpertMagic )) )  // only look if mygrid and symbol...
     {

//--- Here we are closing pending orders only
if ( OrderType() == OP_SELLSTOP )  result = OrderDelete( OrderTicket());

//--- Now we are closing only orders that are have a profit less than are "extern double ProfitValueClose" setting above
if ( OrderType() == OP_SELL && OrderProfit() < ProfitValueClose)  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(),MODE_ASK),5,Red );

//--- If are "extern bool CloseOnSwitch" setting is true, close all open sell orders regardless of profit
if ( OrderType() == OP_SELL && CloseOnSwitch == true)  result = OrderClose( OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(),MODE_ASK),5,Red );

      }
  }
  return;
}



///--- Here we initialize the expert and tell it to start reading 

int start()

{


int i, ticket;

///--- Lets check for positions that are already open or pending
int total = OrdersTotal();


///--- First we'll say our orders and profit are zero and let the Expert calculate the actual numbers from here
double OpenLongOrders = 0, OpenShortOrders = 0, PendingLongOrders =0, PendingShortOrders =0, LongOrderProfit = 0, ShortOrderProfit = 0; 


///--- LTK LongTicket and STK ShortTicket are for pulling last ticket number numbers on the most recent orders we have placed
double LongTicket, ShortTicket;

          
 for( i=0;i<total;i++)       // scan all open orders and positions
   {
     OrderSelect(i, SELECT_BY_POS );
      if ( OrderMagicNumber() == ExpertMagic )  // only look at orders with our expert number attached
         {
           int type = OrderType();

if (( OrderSymbol() == Symbol() && (type == OP_BUY ))) {OpenLongOrders=OpenLongOrders+1; LongOrderProfit = LongOrderProfit + OrderProfit(); LongTicket = OrderTicket( );}
if (( OrderSymbol() == Symbol() && (type == OP_SELL ))) {OpenShortOrders=OpenShortOrders+1; ShortOrderProfit = ShortOrderProfit + OrderProfit(); ShortTicket = OrderTicket();}
if (( OrderSymbol() == Symbol() && (type == OP_BUYSTOP ))) {PendingLongOrders=PendingLongOrders+1;}
if (( OrderSymbol() == Symbol() && (type == OP_SELLSTOP ))) {PendingShortOrders=PendingShortOrders+1;}



bool LTK = false, STK = false; 


///--- Here we are checking to see if our order ticket matches before we execute more trades
///--- By checking the ticket first we can calculate if it's profit is >0. If true, we will allow another similar order to be placed (i.e. a second long or second short order) 
if (( OrderSymbol() == Symbol() && (type == OP_BUY ) && (OrderTicket() == LongTicket) && (OrderProfit()>0))) {LTK = true;}

if (( OrderSymbol() == Symbol() && (type == OP_SELL ) && (OrderTicket() == ShortTicket) && (OrderProfit()>0))) {STK = true;}

                }
              }


///--- Here we have combined our pending and open orders to give us a total of each ones trading pool long and short
/* So if there are 1 pending long + 1 open long the value returned here will be 2  */
double totallongs = (OpenLongOrders+PendingLongOrders);
double totalshorts = (OpenShortOrders+PendingShortOrders);



///--- Here we place our indicators or the features we wish to use to determine our trades.
/* In this example we are using the moving average value of the last bar closed to see if the close was higher that the open or lower
The way we tell the expert to use the last bar closed is the the #1 at the very end. If we wanted the current bar we would change
that number to 0. For two bars back we would change it to the #2 and so on. You can add any amount fof indicators here including
custom indicators using the iCustom feature    */  
double   maclose =iMA(Symbol(),timeFrame1,1,0,MODE_EMA,PRICE_CLOSE,1);
double   maopen  =iMA(Symbol(),timeFrame1,1,0,MODE_EMA,PRICE_OPEN,1);



///--- Here we place our rules for placing orders.
if(maclose>maopen)
{golong = true;}

if(maclose<maopen)
{goshort = true;}


if (totallongs >1)
{golong = false;}

if (totalshorts >1)
{goshort = false;}


///--- More rules are being added here. These are telling us to trade only during certain hours of the day. 
//### SERVER TIMES TO ALLOW NEW ORDERS
double timeallowed = false;

if((UseTimeTrader == true) && ( TimeCurrent() == StrToTime(Start+":00")))
{timeallowed = true;}



///--- More rules are being added here. This one tells the expert to close orders when a certain time of day is hit. 
//### SERVER TIMES TO EXPIRE AND PENDING ORDERS
if((UseTimeExpire == true) && ( TimeCurrent()>= StrToTime(ExpireHour+":00") && TimeCurrent()<=StrToTime(23+":59:59")))
{CloseLongOrders(); CloseShortOrders(); golong = false; goshort = false;}



///--- If our last long order is postive in profit, here we are allowing a second one to be placed
if((StackOrders == true) && (LTK == true) && (PendingLongOrders ==0) && (LongOrderProfit >0) && (maclose>maopen))
{golong = true;}



///--- If our last short order is postive in profit, here we are allowing a second one to be placed
if((StackOrders == true) && (STK == true) && (PendingShortOrders ==0) && (ShortOrderProfit >0) && (maclose<maopen))
{goshort = true;}



///--- When the candle reverses from bullish to bearish, we are telling the expert to close all long orders, and vice versa for shorts
if(maclose>maopen)
{CloseShortOrders();}

if(maclose<maopen)
{CloseLongOrders();}



///--- Our on chart comments are placed here
{ Comment("\n","               ## EXPERT ADVISOR STATS ##", 
"\n" ,"               LONG STATS:  " ,"TradeLongs?   ",golong,"    LongProfits = ",LongOrderProfit, "     OpenLongs=",OpenLongOrders,"   LongTicket# =",LongTicket,
"\n" ,"               SHORT STATS: " ,"TradeShorts?  ",goshort,"   ShortProfits = ",ShortOrderProfit,  "   OpenShorts=", OpenShortOrders, "   ShortTicket# =",ShortTicket);}




///--- Here we tell the expert to trade only according to the time setting we had above
/* We have also asked the expert to not place new trades on Sunday by using DayOfWeek()!=0  "day of week does not =0"
and to not trade on Fridays by using DayOfWeek()!=6   "day of week does not = Friday"  */
if (timeallowed == true && golong==true && DayOfWeek()!=0 && DayOfWeek()!=6)
{ticket=OrderSend(Symbol(),OP_BUYSTOP,LotSize,Ask+10*Point,0,Ask-StopLoss*Point,Ask+(TakeProfit*Point),ExpertName,ExpertMagic,0,Green);

}

if (timeallowed == true && goshort == true && DayOfWeek()!=0 && DayOfWeek()!=6)
{ticket=OrderSend(Symbol(),OP_SELLSTOP,LotSize,Bid-10*Point,0,Bid+StopLoss*Point,Bid-(TakeProfit*Point),ExpertName,ExpertMagic,0,Red);

}


return(0);

}

